/*
 * Default Agar Stylesheet.
 * ex:syn=agarcss
 *
 * Widgets inherit colors and fonts from their parents by default.
 * Margins, paddings and spacings are not inherited. Selectors include:
 *
 *   Selector type                Example
 *   -------------                --------
 *   Widget(s) of class (exact)   AG_Box
 *   Widget(s) of class (pattern) AG_Widget:AG_Box:*
 *   Child widget(s) of class     AG_FontSelector > AG_Tlist
 *   Child widget named           AG_Combo > "trigger"
 *
 * See: AG_StyleSheet(3) and "STYLE ATTRIBUTES" section of AG_Widget(3).
 */

AG_Box {
	padding: 2 3 2 3;    /* top right bottom left */
	spacing: 3 2;        /* horiz vert */
}
AG_Button {
	padding: 5 10 5 10;
	color#focused: #808081;
	color#hover: #868580;
	selection-color#focused: #646496;
	selection-color#hover: #969664;
}
AG_Checkbox {
	spacing: 5;
	background-color#hover: rgb(80,80,120);
}
AG_Combo > "input" {
	padding: 0 0 2 0;
	font-size: 95%;
}
AG_Combo > "trigger" {
	padding: 2;
}
AG_Console {
	font-family: monoalgue;
	padding: 4;
	spacing: 3;
	background-color: #000;
	background-color#disabled: #222;
	text-color: #f0f0f0;
	text-color#disabled: #ccc;
}
AG_Console > AG_Scrollbar {
	font-size: 60%;
	color: #444;
	color#hover: #40404a;
	text-color: #aaa;
}
AG_Editable {
	padding: 1 3 1 3;
	background-color: #0000;
}
AG_FileDlg {
	spacing: 2;
	padding: 3;
}
AG_Icon {
	spacing: 4 0;
}
AG_Label {
	padding: 2 6 2 6;
	background-color: #0000;
	background-color#focused: #0000;
}
AG_Menu {
	padding: 2 5 2 5;
	color: rgb(60,60,60);
	color#disabled: rgb(40,40,110);
	selection-color: rgb(40,40,110);
	text-color#disabled: rgb(170,170,170);
}
AG_Menu (zoom = 0-9) {
	font-family: league-spartan;
	font-size: 140%;
	padding: 1 2 1 2;
}
AG_Menu (zoom > 24) {
	font-family: league-spartan-light;
	padding: 2 5 10 5;
}
AG_MenuView {
	padding: 4 8 4 8;
	spacing: 8 0;
	color: rgb(60,60,60);
	selection-color: rgb(40,40,110);
	color#disabled: rgb(40,40,110);
	text-color#disabled: rgb(170,170,170);
}
AG_Notebook {
	margin: 2;
}
AG_Numerical > AG_UCombo {
	font-size: 80%;
}
AG_Numerical > AG_Button {
	padding: 0;
	font-size: 55%;
	text-color: #bebebe;
	color#hover: #969590;
}
AG_HSVPal {
	font-family: league-gothic;
	font-size: 140%;
}
AG_HSVPal (width < 50) {
	font-family: league-gothic-condensed;
	font-size: 90%;
}
AG_HSVPal (width < 90) {
	font-family: league-gothic-condensed;
	font-size: 120%;
}
AG_Pane {
	color: #666;
	color#hover: #777;
	line-color#hover: rgb(200,200,180);
}
AG_ProgressBar {
	font-family: league-gothic;
	font-size: 90%;
	padding: 2;
}
AG_Radio {
	spacing: 1;
	background-color#hover: rgb(80,80,120);
}
AG_Textbox {
	padding: 3;
	spacing: 5;
}
AG_Titlebar {
	font-size: 90%;
	padding: 3;
	spacing: 1;
	color: rgb(40,50,60);
	color#disabled: rgb(35,35,35);
}
AG_Titlebar > AG_Label {
	background-color: #0000;
	background-color#focused: #0000;
	padding: 2 0 0 2;
}
AG_Titlebar > AG_Button {
	padding: 0 5 0 5;
	color: #0000;
	color#hover: #868580;
}
AG_Toolbar {
	padding: 0;
	spacing: 1;
}
AG_Scrollbar {
	font-size: 60%;
	color: #666;
	color#hover: #777;
	text-color: #bebebe;
}
AG_Separator {
	line-color: #888;
	padding: 8 10 8 10;
}
AG_Statusbar {
	padding: 2;
	spacing: 1;
}
AG_Tlist {
	padding: 1 0 2 0;
	spacing: 4 0;
}
AG_UCombo > "trigger" {
	padding: 3;
}
AG_Window {
	padding: 0 3 4 3;
	background-color: #575757;
}
